@@ -3,7 +3,7 @@
 # License: GPLv3
 
 '''
-uxf is a plain text human readable storage format that may serve as a
+UXF is a plain text human readable storage format that may serve as a
 convenient alternative to csv, ini, json, sqlite, toml, xml, or yaml.
 
 The uxf module can be used as an executable. Run
@@ -12,27 +12,37 @@ The uxf module can be used as an executable. Run
 
 to see the command line help.
 
-uxf's public API provides two functions and three classes.
+uxf's public API provides two functions and five classes.
 
     def read(filename_or_filelike): -> (data, custom_header)
 
-In the returned 2-tuple the data is a dict (i.e., a uxf map), list, or
-uxf.Table, and the custom_header is a (possibly empty) custom string.
+In the returned 2-tuple the data is a Map, List, or Table, and the
+custom_header is a (possibly empty) custom string.
 
     def write(filename_or_filelike, data, custom)
 
 This writes the data (and custom header if supplied) into the given file as
-uxf data. The data must be a single dict, list, or uxf.Table.
+UXF data. The data must be a single Map, dict, List, list, or Table.
 
     class Error
 
 Used to propagate errors (and warnings if warn_is_error is True).
 
+    class List
+
+This is used to represent a UXF list. It is a list subclass that also has a
+.comment attribute.
+
+    class Map
+
+This is used to represent a UXF map. It is a dict subclass that also has a
+.comment attribute.
+
     class Table
 
-Used to store uxf.Tables. A uxf.Table has a list of fieldnames and a records
+Used to store UXF Tables. A Table has a list of fieldnames and a records
 list which is a lists of lists with each sublist having the same number of
-items as the number of fieldnames.
+items as the number of fieldnames. It also has a .comment attribute.
 
     class NTuple
 
@@ -42,8 +52,8 @@ storing complex numbers, points (2D or 3D), IP addresses, and RGB and RGBA
 values.
 
 Note that the __version__ is the module version (i.e., the versio of this
-implementation), while the VERSION is the maximum uxf version that this
-module can read (and the uxf version that it writes).
+implementation), while the VERSION is the maximum UXF version that this
+module can read (and the UXF version that it writes).
 '''
 
 import collections
@@ -60,8 +70,9 @@ except ImportError:
     isoparse = None
 
 
-__all__ = ('__version__', 'VERSION', 'read', 'write', 'Table', 'NTuple')
-__version__ = '0.9.0' # uxf module version
+__all__ = ('__version__', 'VERSION', 'read', 'write', 'List', 'Map',
+           'Table', 'NTuple')
+__version__ = '0.9.4' # uxf module version
 VERSION = 1.0 # uxf file format version
 
 UTF8 = 'utf-8'
@@ -69,9 +80,9 @@ UTF8 = 'utf-8'
 
 def read(filename_or_filelike, *, warn_is_error=False):
     '''
-    Returns a 2-tuple, the first item of which is a dict (i.e., a uxf map),
-    list, or uxf.Table containing all the uxf data read. The second item is
-    the custom string (if any) from the file's header.
+    Returns a 2-tuple, the first item of which is a Map, List, or Table
+    containing all the UXF data read. The second item is the custom string
+    (if any) from the file's header.
 
     filename_or_filelike is sys.stdin or a filename or an open readable file
     (text mode UTF-8 encoded).
@@ -144,19 +155,19 @@ class _Lexer(_ErrorMixin):
     def scan_header(self):
         i = self.text.find('\n')
         if i == -1:
-            self.error('missing uxf file header or empty file')
+            self.error('missing UXF file header or empty file')
         self.pos = i
         parts = self.text[:i].split(None, 2)
         if len(parts) < 2:
-            self.error('invalid uxf file header')
+            self.error('invalid UXF file header')
         if parts[0] != 'uxf':
-            self.error('not a uxf file')
+            self.error('not a UXF file')
         try:
             version = float(parts[1])
             if version > VERSION:
                 self.warn(f'version ({version}) > current ({VERSION})')
         except ValueError:
-            self.warn('failed to read uxf file version number')
+            self.warn('failed to read UXF file version number')
         if len(parts) > 2:
             self.custom = parts[2]
 
@@ -189,9 +200,19 @@ class _Lexer(_ErrorMixin):
         elif c == ']':
             self.add_token(_Kind.LIST_END)
         elif c == '{':
-            self.add_token(_Kind.DICT_BEGIN)
+            self.add_token(_Kind.MAP_BEGIN)
         elif c == '}':
-            self.add_token(_Kind.DICT_END)
+            self.add_token(_Kind.MAP_END)
+        elif c == '#':
+            if self.tokens and self.tokens[-1].kind in {
+                    _Kind.LIST_BEGIN, _Kind.MAP_BEGIN, _Kind.TABLE_BEGIN}:
+                if self.peek() != '<':
+                    self.error('a str must follow the # comment '
+                               f'introducer, got {c!r}')
+                self.read_comment()
+            else:
+                self.error('comments may only occur at the start of maps, '
+                           'lists, and tables')
         elif c == '<':
             self.read_string_or_name()
         elif c == '(':
@@ -210,6 +231,12 @@ class _Lexer(_ErrorMixin):
             self.error(f'invalid character encountered: {c!r}')
 
 
+    def read_comment(self):
+        self.pos += 1 # skip the leading <
+        value = self.match_to('>', error_text='unterminated string or name')
+        self.add_token(_Kind.COMMENT, unescape(value))
+
+
     def read_string_or_name(self):
         value = self.match_to('>', error_text='unterminated string or name')
         self.add_token(self.text_kind, unescape(value))
@@ -219,7 +246,7 @@ class _Lexer(_ErrorMixin):
 
     def read_ntuple(self):
         self.pos += 1 # skip the leading : of (:
-        value = self.match_to(':', error_text='unterminated NTuple')
+        value = self.match_to(':', error_text='unterminated ntuple')
         if self.peek() != ')':
             self.error(f'expected \')\', got {self.peek()!r}')
         self.pos += 1 # skip the trailing ) of :)
@@ -386,8 +413,9 @@ class _Kind(enum.Enum):
     TABLE_END = enum.auto()
     LIST_BEGIN = enum.auto()
     LIST_END = enum.auto()
-    DICT_BEGIN = enum.auto()
-    DICT_END = enum.auto()
+    MAP_BEGIN = enum.auto()
+    MAP_END = enum.auto()
+    COMMENT = enum.auto()
     NTUPLE = enum.auto()
     NULL = enum.auto()
     BOOL = enum.auto()
@@ -401,7 +429,7 @@ class _Kind(enum.Enum):
 
 
 class NTuple:
-    '''Used to store a uxf.NTuple.
+    '''Used to store a UXF ntuple.
 
     A uxf.NTuple holds 2-12 ints or 2-12 floats.
 
@@ -423,10 +451,13 @@ class NTuple:
     >>> nt[3], nt.d, nt.fourth
     (40, 40, 40)
 
-    Every item can be accessed by index 0 <= min(len(), 12) or by fieldname.
-    The first three items can be accessed as x, y, and z, respectively.
-    All items can be accessed as a, b, c, ..., j, k, l, respectively, and as
-    first, second, third, ..., tenth, eleventh, twelth, respectively.
+    Every item can be accessed by index 0 <= min(len(), 11) or by fieldname.
+    The first three items can also be accessed as attributes .x, .y, and .z,
+    respectively. All items can be accessed as attributes .a, .b, .c, ...,
+    .j, .k, .l, respectively, and as .first, .second, .third, ..., .tenth,
+    .eleventh, .twelth, respectively. In addition, for NTuples of length
+    two, the first item can be accessed as attribute .real and the second as
+    .imag or .imaginary.
     '''
 
     __slots__ = ('_items',)
@@ -478,6 +509,11 @@ class NTuple:
             return self._items[10]
         if name in {'l', 'twelth'}:
             return self._items[11]
+        if len(self._items) == 2:
+            if name == 'real':
+                return self._items[0]
+            elif name in {'imag', 'imaginary'}:
+                return self._items[1]
         raise AttributeError(f'{self.__class__.__name__!r} object has '
                              f'no attribute {name!r}')
 
@@ -491,34 +527,64 @@ class NTuple:
         return f'{self.__class__.__name__}({items})'
 
 
+    def __len__(self):
+        return len(self._items)
+
+
+class List(list):
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.comment = None
+
+
+class Map(dict):
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.comment = None
+
+
 class Table:
-    '''Used to store a uxf.Table.
+    '''Used to store a UXF table.
 
-    A uxf.Table has a list of fieldnames and a records list which is a
-    lists of lists with each sublist having the same number of items as
-    the number of fieldnames.
+    A Table has a list of fieldnames and a records list which is a lists of
+    lists with each sublist having the same number of items as the number of
+    fieldnames. It also has a .comment attribute.
 
-    When a uxf.Table is iterated each row is returned as a namedtuple.
+    When a Table is iterated each row is returned as a namedtuple.
     '''
 
-    def __init__(self, *, name=None, fieldnames=None, items=None):
+    def __init__(self, *, name=None, fieldnames=None, records=None,
+                 comment=None):
         '''
-        items can be a flat list of values (which will be put into a list of
-        lists with each sublist being len(fieldnames) long), or a list of
+        A Table may be created empty, e.g., Table(). However, if records is
+        not None, then both name and fieldnames must be given.
+
+        records can be a flat list of values (which will be put into a list
+        of lists with each sublist being len(fieldnames) long), or a list of
         lists in which case each list is _assumed_ to be len(fieldnames)
         long.
+
+        comment is an optional str.
         '''
         self.name = name
         self._Class = None
         self.fieldnames = [] if fieldnames is None else fieldnames
         self.records = []
-        if items:
-            if isinstance(items, list):
+        self.comment = comment
+        if records:
+            if not name:
+                raise Error('can\'t create an unnamed nonempty table')
+            if not fieldnames:
+                raise Error(
+                    'can\'t create a nonempty table without field names')
+            if isinstance(records, list):
                 if self._Class is None:
                     self._make_class()
-                self.records = items
+                self.records = records
             else:
-                for value in items:
+                for value in records:
                     self.append(value)
 
 
@@ -537,9 +603,9 @@ class Table:
 
     def _make_class(self):
         if not self.name:
-            raise Error('can\'t use an unnamed Table')
+            raise Error('can\'t use an unnamed table')
         if not self.fieldnames:
-            raise Error('can\'t create a Table with no field names')
+            raise Error('can\'t create a table with no field names')
         self._Class = collections.namedtuple(
             _canonicalize(self.name, 'Table'),
             [_canonicalize(name, f'Field{i}')
@@ -548,9 +614,9 @@ class Table:
 
     def __iadd__(self, value):
         if not self.name:
-            raise Error('can\'t append to an unnamed Table')
+            raise Error('can\'t append to an unnamed table')
         if not self.fieldnames:
-            raise Error('can\'t append to a Table with no field names')
+            raise Error('can\'t append to a table with no field names')
         if isinstance(value, (list, tuple)): # or: collections.abc.Sequence?
             for v in value:
                 self.append(v)
@@ -572,7 +638,13 @@ class Table:
 
     def __str__(self):
         return (f'Table {self.name!r} {self.fieldnames!r} with '
-                f'{len(self.records)} records')
+                f'{len(self.records)} records #{self.comment!r}')
+
+
+    def __repr__(self):
+        return (f'{self.__class__.__name__}(name={self.name!r}, '
+                f'fieldnames={self.fieldnames!r}, '
+                f'records={self.records!r}, comment={self.comment!r})')
 
 
 def _canonicalize(s, prefix):
@@ -613,10 +685,16 @@ class _Parser(_ErrorMixin):
                 break
             self.pos = token.pos
             state = self.states[-1]
+            if state is _Expect.COMMENT:
+                if token.kind is _Kind.COMMENT:
+                    self.stack[-1].comment = token.value
+                    continue
+                self.states.pop() # Didn't get a comment after all
+                state = self.states[-1]
             if state is _Expect.COLLECTION:
                 if not self._is_collection_start(token.kind):
-                    self.error(f'expected dict (uxf map), list, or '
-                               f'uxf.Table, got {token}')
+                    self.error(f'expected Map, dict, List, list, or '
+                               f'Table, got {token}')
                 self.states.pop() # _Expect.COLLECTION
                 self._on_collection_start(token.kind)
                 data = self.stack[0]
@@ -626,10 +704,10 @@ class _Parser(_ErrorMixin):
                 self._handle_field_name(token)
             elif state is _Expect.TABLE_VALUE:
                 self._handle_table_value(token)
-            elif state is _Expect.DICT_KEY:
-                self._handle_dict_key(token)
-            elif state is _Expect.DICT_VALUE:
-                self._handle_dict_value(token)
+            elif state is _Expect.MAP_KEY:
+                self._handle_map_key(token)
+            elif state is _Expect.MAP_VALUE:
+                self._handle_map_value(token)
             elif state is _Expect.EOF:
                 if token.kind is not _Kind.EOF:
                     self.error(f'expected EOF, got {token}')
@@ -641,28 +719,27 @@ class _Parser(_ErrorMixin):
 
 
     def _is_collection_start(self, kind):
-        return kind in {_Kind.DICT_BEGIN, _Kind.LIST_BEGIN,
+        return kind in {_Kind.MAP_BEGIN, _Kind.LIST_BEGIN,
                         _Kind.TABLE_BEGIN}
 
 
     def _is_collection_end(self, kind):
-        return kind in {_Kind.DICT_END, _Kind.LIST_END,
+        return kind in {_Kind.MAP_END, _Kind.LIST_END,
                         _Kind.TABLE_END}
 
 
     def _on_collection_start(self, kind):
-        if kind is _Kind.DICT_BEGIN:
-            self.states.append(_Expect.DICT_KEY)
-            self._on_collection_start_helper(dict)
+        if kind is _Kind.MAP_BEGIN:
+            self.states += [_Expect.MAP_KEY, _Expect.COMMENT]
+            self._on_collection_start_helper(Map)
         elif kind is _Kind.LIST_BEGIN:
-            self.states.append(_Expect.ANY_VALUE)
-            self._on_collection_start_helper(list)
+            self.states += [_Expect.ANY_VALUE, _Expect.COMMENT]
+            self._on_collection_start_helper(List)
         elif kind is _Kind.TABLE_BEGIN:
-            self.states.append(_Expect.TABLE_NAME)
+            self.states += [_Expect.TABLE_NAME, _Expect.COMMENT]
             self._on_collection_start_helper(Table)
         else:
-            self.error('expected to create dict (uxf map), list, or '
-                       f'uxf.Table, not {kind}')
+            self.error('expected to create Map, List, or Table, not {kind}')
 
 
     def _on_collection_start_helper(self, Class):
@@ -680,7 +757,7 @@ class _Parser(_ErrorMixin):
             if isinstance(self.stack[-1], list):
                 self.states.append(_Expect.ANY_VALUE)
             elif isinstance(self.stack[-1], dict):
-                self.states.append(_Expect.DICT_KEY)
+                self.states.append(_Expect.MAP_KEY)
             elif isinstance(self.stack[-1], Table):
                 self.states.append(_Expect.TABLE_VALUE)
             else:
@@ -691,7 +768,7 @@ class _Parser(_ErrorMixin):
 
     def _handle_table_name(self, token):
         if token.kind is not _Kind.TABLE_NAME:
-            self.error(f'expected Table name, got {token}')
+            self.error(f'expected table name, got {token}')
         self.stack[-1].name = token.value
         self.states[-1] = _Expect.TABLE_FIELD_NAME
 
@@ -701,7 +778,7 @@ class _Parser(_ErrorMixin):
             self.states[-1] = _Expect.TABLE_VALUE
         else:
             if token.kind is not _Kind.TABLE_FIELD_NAME:
-                self.error(f'expected Table field name, got {token}')
+                self.error(f'expected table field name, got {token}')
             self.stack[-1].append_fieldname(token.value)
 
 
@@ -714,49 +791,49 @@ class _Parser(_ErrorMixin):
                 _Kind.STR, _Kind.BYTES}:
             self.stack[-1] += token.value
         else:
-            self.error('Table values may only be null, bool, int, real, '
+            self.error('table values may only be null, bool, int, real, '
                        f'date, datetime, str, or bytes, got {token}')
 
 
-    def _handle_dict_key(self, token):
-        if token.kind is _Kind.DICT_END:
+    def _handle_map_key(self, token):
+        if token.kind is _Kind.MAP_END:
             self._on_collection_end(token)
         elif token.kind in {
                 _Kind.INT, _Kind.DATE, _Kind.DATE_TIME,
                 _Kind.STR, _Kind.BYTES}:
             self.keys.append(token.value)
-            self.states[-1] = _Expect.DICT_VALUE
+            self.states[-1] = _Expect.MAP_VALUE
         else:
-            self.error('dict (uxf map) keys may only be int, date, '
-                       f'datetime, str, or bytes, got {token}')
+            self.error('Map keys may only be int, date, datetime, str, '
+                       f'or bytes, got {token}')
 
 
-    def _handle_dict_value(self, token):
+    def _handle_map_value(self, token):
         if self._is_collection_start(token.kind):
-            # this adds a new list, dict, or Table to the stack
+            # this adds a new list, map, or table to the stack
             self._on_collection_start(token.kind)
-            # this adds a key-value item to the dict that contains the above
-            # list, dict, or Table, the key being the key acquired earlier,
-            # the value being the new list, dict, or Table
+            # this adds a key-value item to the map that contains the above
+            # list, map, or table, the key being the key acquired earlier,
+            # the value being the new list, map, or table
             self.stack[-2][self.keys[-1]] = self.stack[-1]
         elif self._is_collection_end(token.kind):
-            self.states[-1] = _Expect.DICT_KEY
+            self.states[-1] = _Expect.MAP_KEY
             self.stack.pop()
             if self.stack and isinstance(self.stack[-1], dict):
                 self.keys.pop()
         else: # a scalar
-            self.states[-1] = _Expect.DICT_KEY
+            self.states[-1] = _Expect.MAP_KEY
             self.stack[-1][self.keys.pop()] = token.value
 
 
     def _handle_any_value(self, token):
         if self._is_collection_start(token.kind):
-            # this adds a new list, dict, or Table to the stack
+            # this adds a new list, map, or table to the stack
             self._on_collection_start(token.kind)
         elif self._is_collection_end(token.kind):
             self.states.pop()
-            if self.states and self.states[-1] is _Expect.DICT_VALUE:
-                self.states[-1] = _Expect.DICT_KEY
+            if self.states and self.states[-1] is _Expect.MAP_VALUE:
+                self.states[-1] = _Expect.MAP_KEY
             self.stack.pop()
         else: # a scalar
             self.stack[-1].append(token.value)
@@ -765,12 +842,13 @@ class _Parser(_ErrorMixin):
 @enum.unique
 class _Expect(enum.Enum):
     COLLECTION = enum.auto()
-    DICT_KEY = enum.auto()
-    DICT_VALUE = enum.auto()
+    MAP_KEY = enum.auto()
+    MAP_VALUE = enum.auto()
     ANY_VALUE = enum.auto()
     TABLE_NAME = enum.auto()
     TABLE_FIELD_NAME = enum.auto()
     TABLE_VALUE = enum.auto()
+    COMMENT = enum.auto()
     EOF = enum.auto()
 
 
@@ -780,8 +858,8 @@ def write(filename_or_filelike, *, data, custom='', compress=False,
     filename_or_filelike is sys.stdout or a filename or an open writable
     file (text mode UTF-8 encoded).
 
-    data is a list, dict (i.e., uxf map), or uxf.Table that this function
-    will write to the filename_or_filelike in uxf format.
+    data is a Map, dict, List, list, or Table that this function will write
+    to the filename_or_filelike in UXF format.
 
     custom is an optional short user string (with no newlines), e.g., a file
     type description.
@@ -793,7 +871,7 @@ def write(filename_or_filelike, *, data, custom='', compress=False,
 
     Set one_way_conversion to True to convert bytearray items to bytes,
     complex items to uxf.NTuples, and sets, frozensets, tuples, and
-    collections.deques to lists rather than raise an Error.
+    collections.deques to Lists rather than raise an Error.
     '''
     pad = ' ' * indent
     close = False
@@ -826,97 +904,113 @@ class _Writer:
         self.file.write('\n')
 
 
-    def write_value(self, item, indent=0, *, pad, dict_value=False):
+    def write_value(self, item, indent=0, *, pad, map_value=False):
         if isinstance(item, (set, frozenset, tuple, collections.deque)):
             if self.one_way_conversion:
                 item = list(item)
             else:
-                raise Error(f'can only convert {type(item)} to list if '
+                raise Error(f'can only convert {type(item)} to List if '
                             'one_way_conversion is True')
         if isinstance(item, list):
             return self.write_list(item, indent, pad=pad,
-                                   dict_value=dict_value)
+                                   map_value=map_value)
         if isinstance(item, dict):
-            return self.write_dict(item, indent, pad=pad,
-                                   dict_value=dict_value)
+            return self.write_map(item, indent, pad=pad,
+                                  map_value=map_value)
         if isinstance(item, Table):
             return self.write_table(item, indent, pad=pad,
-                                    dict_value=dict_value)
+                                    map_value=map_value)
         return self.write_scalar(item, indent=indent, pad=pad,
-                                 dict_value=dict_value)
+                                 map_value=map_value)
 
 
-    def write_list(self, item, indent=0, *, pad, dict_value=False):
-        tab = '' if dict_value else pad * indent
+    def write_list(self, item, indent=0, *, pad, map_value=False):
+        tab = '' if map_value else pad * indent
+        comment = getattr(item, 'comment', None)
         if len(item) == 0:
-            self.file.write(f'{tab}[]')
-            return False
-        self.file.write(f'{tab}[')
-        indent += 1
-        is_scalar = _is_scalar(item[0])
-        if is_scalar:
-            kwargs = dict(indent=0, pad=' ', dict_value=False)
+            self.file.write(f'{tab}[]\n' if comment is None else
+                            f'{tab}[ #<{escape(comment)}>]\n')
         else:
-            self.file.write('\n')
-            kwargs = dict(indent=indent, pad=pad, dict_value=False)
-        for value in item:
-            self.write_value(value, **kwargs)
+            self.file.write(f'{tab}[')
+            if comment is not None:
+                self.file.write(f' #<{escape(comment)}>')
+            indent += 1
+            is_scalar = _is_scalar(item[0])
             if is_scalar:
-                kwargs['indent'] = 1 # 0 for first item
-        tab = pad * (indent - 1)
-        self.file.write(']\n' if is_scalar else f'{tab}]\n')
+                kwargs = dict(indent=0, pad=' ', map_value=False)
+                if comment is not None:
+                    self.file.write(' ')
+            else:
+                self.file.write('\n')
+                kwargs = dict(indent=indent, pad=pad, map_value=False)
+            for value in item:
+                self.write_value(value, **kwargs)
+                if is_scalar:
+                    kwargs['indent'] = 1 # 0 for first item
+            tab = pad * (indent - 1)
+            self.file.write(']\n' if is_scalar else f'{tab}]\n')
         return True
 
 
-    def write_dict(self, item, indent=0, *, pad, dict_value=False):
-        tab = '' if dict_value else pad * indent
+    def write_map(self, item, indent=0, *, pad, map_value=False):
+        tab = '' if map_value else pad * indent
+        comment = getattr(item, 'comment', None)
         if len(item) == 0:
-            self.file.write(f'{tab}{{}}')
-            return False
+            self.file.write(f'{tab}{{}}\n' if comment is None else
+                            f'{tab}{{ #<{escape(comment)}> }}\n')
         elif len(item) == 1:
             self.file.write(f'{tab}{{')
+            if comment is not None:
+                self.file.write(f' #<{escape(comment)}>')
             key, value = list(item.items())[0]
             self.write_scalar(key, 1, pad=' ')
             self.file.write(' ')
-            self.write_value(value, 1, pad=' ', dict_value=True)
-            self.file.write('}}')
-            return False
-        self.file.write(f'{tab}{{\n')
-        indent += 1
-        for key, value in item.items():
-            self.write_scalar(key, indent, pad=pad)
-            self.file.write(' ')
-            if not self.write_value(value, indent, pad=pad,
-                                    dict_value=True):
-                self.file.write('\n')
-        tab = pad * (indent - 1)
-        self.file.write(f'{tab}}}\n')
+            self.write_value(value, 1, pad=' ', map_value=True)
+            self.file.write('}\n')
+        else:
+            self.file.write(f'{tab}{{\n' if comment is None else
+                            f'{tab}{{ #<{escape(comment)}>\n')
+            indent += 1
+            for key, value in item.items():
+                self.write_scalar(key, indent, pad=pad)
+                self.file.write(' ')
+                if not self.write_value(value, indent, pad=pad,
+                                        map_value=True):
+                    self.file.write('\n')
+            tab = pad * (indent - 1)
+            self.file.write(f'{tab}}}\n')
         return True
 
 
-    def write_table(self, item, indent=0, *, pad, dict_value=False):
-        tab = '' if dict_value else pad * indent
-        self.file.write(f'{tab}[= <{escape(item.name)}>')
+    def write_table(self, item, indent=0, *, pad, map_value=False):
+        tab = '' if map_value else pad * indent
+        comment = getattr(item, 'comment', None)
+        self.file.write(f'{tab}[= ')
+        if comment is not None:
+            self.file.write(f'#<{escape(comment)}> ')
+        self.file.write(f'<{escape(item.name)}>')
         for name in item.fieldnames:
             self.file.write(f' <{escape(name)}>')
         if len(item) == 0:
-            self.file.write(' = =]')
-            return False
-        self.file.write(' =\n')
-        indent += 1
-        for record in item:
-            self.file.write(pad * indent)
-            for value in record:
-                self.write_scalar(value, pad=pad)
-                self.file.write(' ')
-            self.file.write('\n')
-        tab = pad * (indent - 1)
-        self.file.write(f'{tab}=]\n')
+            self.file.write(' = =]\n')
+        else:
+            self.file.write(' =\n')
+            indent += 1
+            for record in item:
+                self.file.write(pad * indent)
+                sep = ''
+                for value in record:
+                    self.file.write(sep)
+                    self.write_scalar(value, pad=pad)
+                    sep = ' '
+                self.file.write('\n')
+            tab = pad * (indent - 1)
+            self.file.write(f'{tab}=]\n')
         return True
 
 
-    def write_scalar(self, item, indent=0, *, pad, dict_value=False):
-        if not dict_value:
+    def write_scalar(self, item, indent=0, *, pad, map_value=False):
+        if not map_value:
             self.file.write(pad * indent)
         if item is None:
             self.file.write('null')
@@ -939,7 +1033,7 @@ class _Writer:
             self.file.write(f'({item.hex().upper()})')
         elif isinstance(item, complex):
             if not self.one_way_conversion:
-                raise Error('can only convert complex to NTuple if '
+                raise Error('can only convert complex to ntuple if '
                             'one_way_conversion is True')
             self.file.write(
                 f'(:{_realstr(item.real)} {_realstr(item.imag)}:)')
